import * as React from "react";
import { Observable } from "rxjs";
import { Store } from "../src/store";
import { ActionMap } from "./actions";
export declare type ExtractProps<TComponentOrTProps> = TComponentOrTProps extends React.ComponentType<infer TProps> ? TProps : TComponentOrTProps;
export interface ConnectResult<TAppState, TOriginalProps> {
    props?: Observable<TOriginalProps>;
    actionMap?: ActionMap<TOriginalProps>;
}
export declare type ConnectCallback<S, P, TInputProps> = (store: Store<S>, inputProps: Observable<TInputProps>) => ConnectResult<S, P>;
export interface ConnectState<TOriginalProps> {
    connectedProps?: Partial<TOriginalProps>;
    ready: boolean;
}
/**
 * Connects a Component's props to a set of props of the application state coming from a Store object.
 */
export declare function connect<TAppState, TOriginalProps extends {}, TInputProps extends {} = {}>(ComponentToConnect: React.ComponentType<TOriginalProps>, connectCallback: ConnectCallback<TAppState, Partial<TOriginalProps>, TInputProps>): {
    new (props: any): {
        render(): JSX.Element;
        context: any;
        setState<K extends "connectedProps" | "ready">(state: ConnectState<TOriginalProps> | ((prevState: Readonly<ConnectState<TOriginalProps>>, props: Readonly<TInputProps & Partial<TOriginalProps>>) => ConnectState<TOriginalProps> | Pick<ConnectState<TOriginalProps>, K> | null) | Pick<ConnectState<TOriginalProps>, K> | null, callback?: (() => void) | undefined): void;
        forceUpdate(callback?: (() => void) | undefined): void;
        readonly props: Readonly<TInputProps & Partial<TOriginalProps>> & Readonly<{
            children?: React.ReactNode;
        }>;
        state: Readonly<ConnectState<TOriginalProps>>;
        refs: {
            [key: string]: React.ReactInstance;
        };
        componentDidMount?(): void;
        shouldComponentUpdate?(nextProps: Readonly<TInputProps & Partial<TOriginalProps>>, nextState: Readonly<ConnectState<TOriginalProps>>, nextContext: any): boolean;
        componentWillUnmount?(): void;
        componentDidCatch?(error: Error, errorInfo: React.ErrorInfo): void;
        getSnapshotBeforeUpdate?(prevProps: Readonly<TInputProps & Partial<TOriginalProps>>, prevState: Readonly<ConnectState<TOriginalProps>>): any;
        componentDidUpdate?(prevProps: Readonly<TInputProps & Partial<TOriginalProps>>, prevState: Readonly<ConnectState<TOriginalProps>>, snapshot?: any): void;
        componentWillMount?(): void;
        UNSAFE_componentWillMount?(): void;
        componentWillReceiveProps?(nextProps: Readonly<TInputProps & Partial<TOriginalProps>>, nextContext: any): void;
        UNSAFE_componentWillReceiveProps?(nextProps: Readonly<TInputProps & Partial<TOriginalProps>>, nextContext: any): void;
        componentWillUpdate?(nextProps: Readonly<TInputProps & Partial<TOriginalProps>>, nextState: Readonly<ConnectState<TOriginalProps>>, nextContext: any): void;
        UNSAFE_componentWillUpdate?(nextProps: Readonly<TInputProps & Partial<TOriginalProps>>, nextState: Readonly<ConnectState<TOriginalProps>>, nextContext: any): void;
    };
    contextType?: React.Context<any> | undefined;
};
